import pino from 'pino';
import * as path from 'path';
import { Module } from '@nestjs/common';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { UsersService } from './users/users.service';
import { TypeOrmModule } from '@nestjs/typeorm';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { DatabaseOptions } from './database-config/database.options';
import { UsersModule } from './users/users.module';
import { UserEntity } from './users/entities/users.entity';
import { AuthModule } from './auth/auth.module';
import { LoggerModule } from 'nestjs-pino';
import { serialize_token, serialize_user_short, serialize_user_long, serialize_res, serialize_req, serialize_job } from './logging.serializers';
import { BooksModule } from './books/books.module';
import { ProvidersModule } from './providers/providers.module';
import { SeriesModule } from './series/series.module';
import { LibraryModule } from './library/library.module';
import { BullModule } from '@nestjs/bullmq';
import { AssetModule } from './asset/asset.module';

@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forRootAsync({
      imports: [ConfigModule],
      useClass: DatabaseOptions
    }),
    BullModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: (config: ConfigService) => ({
        connection: {
          host: config.get('REDIS_HOST') ?? 'localhost',
          port: config.get('REDIS_PORT') ?? 6379,
          password: config.get('REDIS_PASSWORD'),
        }
      })
    }),
    TypeOrmModule.forFeature([UserEntity]),
    UsersModule,
    AuthModule,
    LoggerModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (config: ConfigService) => {
        return {
          pinoHttp: {
            level: config.get('LOG_LEVEL') ?? 'info',
            autoLogging: true,
            serializers: config.get('ENVIRONMENT', 'production') == 'development' ? {
              user: value => serialize_user_long(value),
              access_token: value => serialize_token(value),
              refresh_token: value => serialize_token(value),
              job: value => serialize_job(value),
              req: value => serialize_req(value),
              res: value => serialize_res(value),
            } : {
              user: value => serialize_user_short(value),
              access_token: value => serialize_token(value),
              refresh_token: value => serialize_token(value),
              job: value => serialize_job(value),
              req: value => serialize_req(value),
              res: value => serialize_res(value),
            },
            stream: pino.destination({
              dest: path.join(config.get('LOG_DIRECTORY') ?? 'logs', 'backend.api.log'),
              minLength: 512,
              sync: false,
            }),
          }
        }
      }
    }),
    BooksModule,
    ProvidersModule,
    SeriesModule,
    LibraryModule,
    ConfigModule,
    AssetModule,
  ],
  controllers: [AppController],
  providers: [AppService, UsersService],
})
export class AppModule { }
