import { Injectable, UnauthorizedException } from '@nestjs/common';
import { UserEntity } from 'src/users/entities/users.entity';
import { UsersService } from 'src/users/users.service';
import { AuthRefreshService } from './auth.refresh.service';
import { AuthAccessService } from './auth.access.service';
import { UUID } from 'crypto';
import { AuthenticationDto } from './dto/authentication.dto';
import { LoginDto } from './dto/login.dto';
import { AccessTokenDto } from './dto/access-token.dto';
import { TokenExpiredError } from '@nestjs/jwt';

@Injectable()
export class AuthService {
  constructor(
    private accessTokens: AuthAccessService,
    private refreshTokens: AuthRefreshService,
    private users: UsersService,
  ) { }


  async login(
    loginDetails: LoginDto
  ): Promise<AuthenticationDto> {
    const user = await this.users.findOne(loginDetails);
    if (!user) {
      throw new UnauthorizedException();
    }

    const access_token = await this.accessTokens.generate(user);

    if (!loginDetails.remember_me) {
      return {
        ...access_token,
        refresh_token: null,
        refresh_exp: null,
      }
    }

    const refresh_token = await this.refreshTokens.generate(user);
    return {
      ...access_token,
      refresh_token: refresh_token.refresh_token,
      refresh_exp: refresh_token.exp,
    }
  }

  async renew(
    user: UserEntity,
  ): Promise<AccessTokenDto> {
    return await this.accessTokens.generate(user);
  }

  async validate(
    username: string,
    password: string,
  ): Promise<UserEntity | null> {
    return await this.users.findOne({ user_login: username, password, remember_me: false });
  }

  async verify(
    accessToken: string,
    refreshToken: string
  ): Promise<{ validation: boolean, userId: UUID | null, username: string | null }> {
    let access: any = null;
    let refresh: any = null;

    if (accessToken) {
      try {
        access = await this.accessTokens.verify(accessToken);
      } catch (err) {
        if (!(err instanceof TokenExpiredError)) {
          return {
            validation: false,
            userId: null,
            username: null,
          };
        }
      }
      if (access && (!access.username || !access.sub)) {
        return {
          validation: false,
          userId: null,
          username: null,
        };
      }
    }

    if (refreshToken) {
      try {
        refresh = await this.refreshTokens.verify(refreshToken);
      } catch (err) {
        return {
          validation: false,
          userId: null,
          username: null,
        };
      }

      if (!refresh.username || !refresh.sub) {
        return {
          validation: false,
          userId: null,
          username: null,
        };
      }
    }

    if (!access && !refresh) {
      return {
        validation: false,
        userId: null,
        username: null,
      };
    } else if (!access && refresh) {
      return {
        validation: null,
        userId: null,
        username: null,
      };
    } else if (access && refresh) {
      if (access.username != refresh.username || access.sub != refresh.sub) {
        return {
          validation: false,
          userId: null,
          username: null,
        };
      }
    }

    return {
      validation: true,
      userId: (access ?? refresh).sub,
      username: (access ?? refresh).username,
    };
  }

  async revoke(
    userId: UUID,
    refreshToken: string
  ): Promise<boolean> {
    const res = await this.refreshTokens.revoke(userId, refreshToken);
    return res?.affected === 1;
  }
}
