import { SearchContext } from "./search.context";

export class GoogleSearchContext extends SearchContext {
  constructor(searchQuery: string, params: { [key: string]: string }) {
    super('google', searchQuery, params);
  }


  generateQueryParams(): string {
    const filterParams = ['maxResults', 'startIndex', 'orderBy', 'langRestrict'];
    const searchParams = ['intitle', 'inauthor', 'inpublisher', 'subject', 'isbn'];

    const queryParams = filterParams
      .map(p => p in this.params ? p + '=' + this.params[p] : undefined)
      .filter(p => p !== undefined)
      .join('&');

    const search = this.search.trim();
    const searchQueryParam = [
      search.length > 0 ? search + ' ' : '',
      ...searchParams.map(p => this.params[p] ? p + ':"' + this.params[p] + '"' : ''),
    ].filter(p => p.length > 0).join('');

    return [queryParams, 'q=' + searchQueryParam].filter(q => q.length > 2).join('&');
  }

  get orderBy(): 'newest' | 'relevant' {
    return this.params['orderBy'] as 'newest' | 'relevant' ?? 'relevant';
  }

  set orderBy(value: 'newest' | 'relevant' | null) {
    if (!value) {
      delete this.params['orderBy'];
    } else {
      this.params['orderBy'] = value;
    }
  }

  get maxResults(): number {
    return 'maxResults' in this.params ? parseInt(this.params['maxResults']) : 10;
  }

  set maxResults(value: string) {
    if (!value || isNaN(parseInt(value))) {
      return;
    }

    this.params['maxResults'] = value;
  }

  get startIndex(): number {
    return 'startIndex' in this.params ? parseInt(this.params['startIndex']) : 10;
  }

  set startIndex(value: string) {
    if (!value || isNaN(parseInt(value))) {
      return;
    }

    this.params['startIndex'] = value;
  }

  get intitle(): string {
    return 'intitle' in this.params ? this.params['intitle'] : null;
  }

  set intitle(value: string) {
    if (!value) {
      delete this.params['intitle'];
    } else {
      this.params['intitle'] = value;
    }
  }

  get inpublisher(): string {
    return 'inpublisher' in this.params ? this.params['inpublisher'] : null;
  }

  set inpublisher(value: string) {
    if (!value) {
      delete this.params['inpublisher'];
    } else {
      this.params['inpublisher'] = value;
    }
  }

  get inauthor(): string {
    return 'inauthor' in this.params ? this.params['inauthor'] : null;
  }

  set inauthor(value: string) {
    if (!value) {
      delete this.params['inauthor'];
    } else {
      this.params['inauthor'] = value;
    }
  }

  get isbn(): string {
    return 'isbn' in this.params ? this.params['isbn'] : null;
  }

  set isbn(value: string) {
    if (!value) {
      delete this.params['isbn'];
    } else {
      this.params['isbn'] = value;
    }
  }

  get subject(): string {
    return 'subject' in this.params ? this.params['subject'] : null;
  }

  set subject(value: string) {
    if (!value) {
      delete this.params['subject'];
    } else {
      this.params['subject'] = value;
    }
  }

  previous(pageCount: number = 1): GoogleSearchContext {
    if (pageCount > 0)
      return this.update(-pageCount);
    return this;
  }

  next(pageCount: number = 1): GoogleSearchContext {
    if (pageCount > 0)
      return this.update(pageCount);
    return this;
  }

  private update(pageChange: number): GoogleSearchContext {
    const resultsPerPage = this.params['maxResults'] ? parseInt(this.params['maxResults']) : 10;
    const index = this.params['startIndex'] ? parseInt(this.params['startIndex']) : 0;

    const data = { ...this.params };
    data['startIndex'] = Math.max(0, index + resultsPerPage * pageChange).toString();

    return new GoogleSearchContext(this.search, data);
  }
}