import { Component, ElementRef, inject, NgZone, OnDestroy, signal, ViewChild } from '@angular/core';
import { ReactiveFormsModule } from '@angular/forms';
import { BehaviorSubject, combineLatest, debounceTime, distinctUntilChanged, filter, map, scan, Subscription, tap, throttleTime } from 'rxjs';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { BookSearchResultDto } from '../../shared/dto/book-search-result.dto';
import { MediaSearchItemComponent } from '../media-search-item/media-search-item.component';
import { SearchBoxComponent } from "../search-box/search-box.component";
import { SearchContextDto } from '../../shared/dto/search-context.dto';

const DEFAULT_MAX_RESULTS = 10;

const PROVIDER_SETTINGS = {
  google: {
    FilterSearchParams: ['inauthor', 'inpublisher', 'intitle', 'isbn', 'subject'],
    FilterNoUpdateParams: ['maxResults'],
  }
}

@Component({
  selector: 'add-new-page',
  standalone: true,
  imports: [
    MediaSearchItemComponent,
    ReactiveFormsModule,
    SearchBoxComponent
  ],
  templateUrl: './add-new-page.component.html',
  styleUrl: './add-new-page.component.css'
})
export class AddNewPageComponent implements OnDestroy {
  private readonly _http = inject(HttpClient);
  private readonly _subscriptions: Subscription[] = [];
  private readonly _zone = inject(NgZone);

  @ViewChild('scrollbar') private readonly searchContentRef: ElementRef<Element> = {} as ElementRef;

  search = new BehaviorSubject<string>('');
  filters = new BehaviorSubject<SearchContextDto>(new SearchContextDto());
  page = new BehaviorSubject<number>(0);
  results: BookSearchResultDto[] = [];
  resultsPerPage = signal<number>(10);
  busy = signal<boolean>(false);
  endOfResults = signal<boolean>(false);
  searchError = signal<string | null>(null);

  constructor() {
    this._zone.runOutsideAngular(() => {
      // Subscription for max results.
      this._subscriptions.push(
        this.filters.pipe(
          map(filters => 'maxResults' in filters.values ? parseInt(filters.values['maxResults']) : DEFAULT_MAX_RESULTS)
        ).subscribe(maxResults => {
          this.resultsPerPage.set(maxResults);
        })
      );

      // Subscription for the search bar.
      this._subscriptions.push(
        combineLatest({
          search: this.search.pipe(
            filter(value => value != null),
          ),
          filters: this.filters.pipe(
            map(filters => ({ values: { ...filters.values } }))
          ),
          page: this.page.pipe(
            throttleTime(3000, undefined, { leading: false, trailing: true }),
          ),
        }).pipe(
          debounceTime(1000),
          filter(entry => entry.search.length > 1 || this.isUsingSearchParamsInFilters(entry.filters)),
          scan((acc, next) => {
            // Different search or filters means resetting to page 0.
            const searchChanged = acc.search != next.search && next.search && next.search.length > 1;
            const filtersChanged = this.hasFiltersMismatched(acc.filters, next.filters);
            if (searchChanged || filtersChanged) {
              this.results = [];
              return {
                ...next,
                page: 0,
              };
            }

            // Ignore further page searching if:
            // - there are no more results;
            // - user is still busy loading new pages;
            // - only max results filter changed.
            if (this.endOfResults() || this.busy() || acc.filters.values['maxResults'] != next.filters.values['maxResults']) {
              return {
                ...next,
                page: -1,
              };
            }

            // Keep searching the same page until error stops.
            if (this.searchError() != null) {
              return acc;
            }

            // Next page.
            return {
              ...next,
              page: Math.min(acc.page + 1, next.page),
            };
          }),
          filter(entry => entry.page >= 0),
          distinctUntilChanged(),
        ).subscribe((entry) => {
          this.busy.set(true);
          this.endOfResults.set(false);
          if (this.searchContentRef) {
            this.searchContentRef.nativeElement.scrollTop = 0;
          }

          this._http.get('/api/providers/search',
            {
              params: {
                ...entry.filters.values,
                provider: 'google',
                search: entry.search!,
                startIndex: entry.page * this.resultsPerPage(),
              },
            }
          ).subscribe({
            next: (results: any) => {
              [].push.apply(this.results, results);

              if (results.length < this.resultsPerPage()) {
                this.endOfResults.set(true);
              }
              this.searchError.set(null);
              this.busy.set(false);
            },
            error: (err) => {
              this.busy.set(false);
              if (err instanceof HttpErrorResponse) {
                if (err.status == 400) {
                  this.searchError.set('Something went wrong when Google received the request.');
                } else if (err.status == 401) {
                  this.searchError.set('Unauthorized. Refresh the page to login again.');
                } else if (err.status == 429) {
                  this.searchError.set('Too many requests. Try again in a minute.');
                } else {
                  this.searchError.set(err.name + ': ' + err.message);
                }
              }
            }
          });
        })
      );
    });
  }

  ngOnDestroy(): void {
    this._subscriptions.forEach(s => s.unsubscribe());
  }

  onResultsScroll(event: any): void {
    const scroll = event.target;
    const limit = scroll.scrollHeight - scroll.clientHeight;

    // Prevent page changes when:
    //  - new search is happening (emptied results);
    //  - still scrolling through current content.
    if (scroll.scrollTop == 0 || scroll.scrollTop < limit - 25) {
      return;
    }

    this.page.next(this.page.getValue() + 1);
  }

  private hasFiltersMismatched(prev: SearchContextDto, next: SearchContextDto) {
    if (prev == next) {
      return false;
    }

    for (let key in prev.values) {
      if (PROVIDER_SETTINGS.google.FilterNoUpdateParams.includes(key))
        continue;
      if (!(key in next.values))
        return true;
      if (prev.values[key] != next.values[key])
        return true;
    }

    for (let key in next.values) {
      if (!PROVIDER_SETTINGS.google.FilterNoUpdateParams.includes(key) && !(key in prev.values))
        return true;
    }

    return false;
  }

  private isUsingSearchParamsInFilters(context: SearchContextDto) {
    if (!context)
      return false;

    const keys = Object.keys(context.values);
    return keys.some(key => PROVIDER_SETTINGS.google.FilterSearchParams.includes(key));
  }
}
