import { AfterViewInit, Component, ElementRef, inject, NgZone, OnDestroy, output, ViewChild } from '@angular/core';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { SearchContextDto } from '../../shared/dto/search-context.dto';
import { filter, map, Subscription } from 'rxjs';
import { ConfigService } from '../../services/config.service';
import { CommonModule } from '@angular/common';

@Component({
  selector: 'search-box',
  standalone: true,
  imports: [
    CommonModule,
    ReactiveFormsModule,
  ],
  templateUrl: './search-box.component.html',
  styleUrl: './search-box.component.css'
})
export class SearchBoxComponent implements AfterViewInit, OnDestroy {
  private readonly _subscriptions: Subscription[] = [];
  private readonly _zone = inject(NgZone);

  @ViewChild('languageSelect') private readonly languageRef: ElementRef<HTMLSelectElement> = {} as ElementRef;
  @ViewChild('orderBySelect') private readonly orderByRef: ElementRef<HTMLSelectElement> = {} as ElementRef;
  @ViewChild('resultsSizeSelect') private readonly resultsSizeRef: ElementRef<HTMLSelectElement> = {} as ElementRef;
  @ViewChild('authorInput') private readonly authorRef: ElementRef<HTMLInputElement> = {} as ElementRef;
  @ViewChild('isbnInput') private readonly isbnRef: ElementRef<HTMLInputElement> = {} as ElementRef;

  config = inject(ConfigService).config;
  filtersEnabled = new FormControl<boolean>(false);
  search = new FormControl<string>('');
  searchOutput = output<string>();
  filters = new SearchContextDto();
  filtersOutput = output<SearchContextDto>();
  isbn = new FormControl<string>('');
  author = new FormControl<string>('');

  constructor() {
    this._zone.runOutsideAngular(() => {
      this._subscriptions.push(
        this.search.valueChanges.pipe(
          filter(value => value != null),
          map(value => value!.trim()),
          filter(value => value.length > 0),
        ).subscribe((value) => this.searchOutput.emit(value!))
      );

      this._subscriptions.push(
        this.author.valueChanges.pipe(
          filter(value => value != null),
          map(value => value!.trim()),
          filter(value => value.length > 0),
        ).subscribe((value) => this.updateFilters('inauthor', { value: value }))
      );

      this._subscriptions.push(
        this.isbn.valueChanges.pipe(
          filter(value => value != null),
          map(value => value!.trim()),
          map(value => value.length == 10 || value.length >= 13 && value.length <= 15 ? value : ''),
          filter(value => value.length > 0),
        ).subscribe((value) => this.updateFilters('isbn', { value: value }))
      );
    });
  }

  ngAfterViewInit() {
    if (this.languageRef && this.languageRef.nativeElement.value)
      this.updateFilters('langRestrict', this.languageRef.nativeElement);
    if (this.orderByRef && this.orderByRef.nativeElement.value)
      this.updateFilters('orderBy', this.orderByRef.nativeElement);
    if (this.resultsSizeRef && this.resultsSizeRef.nativeElement.value)
      this.updateFilters('maxResults', this.resultsSizeRef.nativeElement);
    if (this.authorRef && this.authorRef.nativeElement.value)
      this.updateFilters('inauthor', this.authorRef.nativeElement);
    if (this.isbnRef && this.isbnRef.nativeElement.value)
      this.updateFilters('isbn', this.isbnRef.nativeElement);
  }

  ngOnDestroy(): void {
    this._subscriptions.forEach(s => s.unsubscribe());
  }

  get provider() {
    switch (this.config.providers.default) {
      case 'google': return this.config.providers.google;
      default: return this.config.providers.google;
    }
  }

  updateFilters(key: string, value: any) {
    if (!value) {
      delete this.filters.values[key];
    } else {
      this.filters.values[key] = value.value;
    }
    this.filtersOutput.emit(this.filters);
  }
}
