import { HTTP_INTERCEPTORS, HttpErrorResponse, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from "@angular/common/http";
import { inject, Injectable } from "@angular/core";
import { BehaviorSubject, catchError, filter, Observable, switchMap, take, throwError } from "rxjs";
import { AuthService } from "../../services/auth/auth.service";

@Injectable()
class TokenValidationHttpInterceptor implements HttpInterceptor {
  private readonly _auth = inject(AuthService);
  private readonly _filters: { url: string, methods: string[] | null }[] = [
    {
      url: '/api/auth/validate',
      methods: null,
    },
    {
      url: '/api/asset/config',
      methods: null,
    },
    {
      url: '/api/auth/login',
      methods: null,
    },
    {
      url: '/api/auth/register',
      methods: null,
    },
  ];

  private _isRefreshing = false;
  private _refreshSubject = new BehaviorSubject<boolean>(false);


  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    let uri = request.url;
    if (!request.url.startsWith('/') && !request.url.startsWith('http')) {
      uri = '/' + request.url;
    }

    const method = request.method.toUpperCase();
    if (this._filters.some(item => uri.startsWith(item.url) && (!item.methods || item.methods.includes(method)))) {
      return next.handle(request);
    }

    return next.handle(request)
      .pipe(
        catchError((err) => {
          if (err instanceof HttpErrorResponse && err.status == 401) {
            return this.handleTokenRenewal(request, next);
          }
          return throwError(() => err);
        }),
      );
  }

  private handleTokenRenewal(request: HttpRequest<any>, next: HttpHandler) {
    if (!this._isRefreshing) {
      this._isRefreshing = true;
      this._refreshSubject.next(false);

      return this._auth.renew()
        .pipe(
          take(1),
          switchMap((token: any) => {
            this._isRefreshing = false;
            this._refreshSubject.next(true);
            return next.handle(request.clone())
          })
        );
    }

    return this._refreshSubject.pipe(
      filter(token => token != null),
      take(1),
      switchMap(_ => {
        return next.handle(request.clone());
      })
    );
  }
}

export const TokenValidationInterceptor = [
  {
    provide: HTTP_INTERCEPTORS,
    useClass: TokenValidationHttpInterceptor,
    multi: true,
  }
];